@@ -40,8 +40,14 @@ from filelock import FileLock, Timeout
 from ansiblelint import cli
 from ansiblelint._mockings import _perform_mockings_cleanup
 from ansiblelint.app import get_app
-from ansiblelint.color import console, console_options, reconfigure, render_yaml
-from ansiblelint.config import options
+from ansiblelint.color import (
+    console,
+    console_options,
+    console_stderr,
+    reconfigure,
+    render_yaml,
+)
+from ansiblelint.config import get_version_warning, options
 from ansiblelint.constants import EXIT_CONTROL_C_RC, LOCK_TIMEOUT_RC
 from ansiblelint.file_utils import abspath, cwd, normpath
 from ansiblelint.skip_utils import normalize_tag
@@ -88,10 +94,6 @@ def initialize_options(arguments: list[str] | None = None) -> None:
     new_options = cli.get_config(arguments or [])
     new_options.cwd = pathlib.Path.cwd()
 
-    if new_options.version:
-        print(f"ansible-lint {__version__} using ansible {ansible_version()}")
-        sys.exit(0)
-
     if new_options.colored is None:
         new_options.colored = should_do_markup()
 
@@ -115,14 +117,14 @@ def initialize_options(arguments: list[str] | None = None) -> None:
         options.cache_dir_lock = FileLock(f"{options.cache_dir}/.lock")
         try:
             options.cache_dir_lock.acquire(timeout=180)
-        except Timeout:
+        except Timeout:  # pragma: no cover
             _logger.error(
                 "Timeout waiting for another instance of ansible-lint to release the lock."
             )
             sys.exit(LOCK_TIMEOUT_RC)
 
     # Avoid extra output noise from Ansible about using devel versions
-    if "ANSIBLE_DEVEL_WARNING" not in os.environ:
+    if "ANSIBLE_DEVEL_WARNING" not in os.environ:  # pragma: no branch
         os.environ["ANSIBLE_DEVEL_WARNING"] = "false"
 
 
@@ -136,7 +138,7 @@ def _do_list(rules: RulesCollection) -> int:
         rules_as_str,
     )
 
-    if options.listrules:
+    if options.list_rules:
 
         _rule_format_map: dict[str, Callable[..., Any]] = {
             "plain": rules_as_str,
@@ -148,8 +150,8 @@ def _do_list(rules: RulesCollection) -> int:
         console.print(_rule_format_map[options.format](rules), highlight=False)
         return 0
 
-    if options.listtags:
-        console.print(render_yaml(rules.listtags()))
+    if options.list_tags:
+        console.print(render_yaml(rules.list_tags()))
         return 0
 
     # we should not get here!
@@ -174,7 +176,17 @@ def _do_transform(result: LintResult, opts: Namespace) -> None:
     transformer.run()
 
 
-# pylint: disable=too-many-branches
+def support_banner() -> None:
+    """Display support banner when running on unsupported platform."""
+    if sys.version_info < (3, 9, 0):
+        prefix = "::warning::" if "GITHUB_ACTION" in os.environ else "WARNING: "
+        console_stderr.print(
+            f"{prefix}ansible-lint is no longer tested under Python {sys.version_info.major}.{sys.version_info.minor} and will soon require 3.9. Do not report bugs for this version.",
+            style="bold red",
+        )
+
+
+# pylint: disable=too-many-branches,too-many-statements
 def main(argv: list[str] | None = None) -> int:  # noqa: C901
     """Linter CLI entry point."""
     # alter PATH if needed (venv support)
@@ -187,26 +199,44 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
     console_options["force_terminal"] = options.colored
     reconfigure(console_options)
 
+    if options.version:
+        console.print(
+            f"ansible-lint [repr.number]{__version__}[/] using ansible [repr.number]{ansible_version()}[/]"
+        )
+        msg = get_version_warning()
+        if msg:
+            console.print(msg)
+        support_banner()
+        sys.exit(0)
+    else:
+        support_banner()
+
     initialize_logger(options.verbosity)
     _logger.debug("Options: %s", options)
     _logger.debug(os.getcwd())
 
-    app = get_app(offline=options.offline)
+    if not options.offline:
+        # pylint: disable=import-outside-toplevel
+        from ansiblelint.schemas import refresh_schemas
+
+        refresh_schemas()
+
     # pylint: disable=import-outside-toplevel
     from ansiblelint.rules import RulesCollection
     from ansiblelint.runner import _get_matches
 
-    rules = RulesCollection(options.rulesdirs, profile=options.profile)
+    rules = RulesCollection(options.rulesdirs, profile_name=options.profile)
 
-    if options.profile == []:
+    if options.list_profiles:
         from ansiblelint.generate_docs import profiles_as_rich
 
         console.print(profiles_as_rich())
         return 0
 
-    if options.listrules or options.listtags:
+    if options.list_rules or options.list_tags:
         return _do_list(rules)
 
+    app = get_app()
     if isinstance(options.tags, str):
         options.tags = options.tags.split(",")
     result = _get_matches(rules, options)
@@ -314,11 +344,11 @@ def _run_cli_entrypoint() -> None:
         sys.exit(main(sys.argv))
     except OSError as exc:
         # NOTE: Only "broken pipe" is acceptable to ignore
-        if exc.errno != errno.EPIPE:
+        if exc.errno != errno.EPIPE:  # pragma: no cover
             raise
-    except KeyboardInterrupt:
+    except KeyboardInterrupt:  # pragma: no cover
         sys.exit(EXIT_CONTROL_C_RC)
-    except RuntimeError as exc:
+    except RuntimeError as exc:  # pragma: no cover
         raise SystemExit(exc) from exc
 
 
@@ -331,6 +361,19 @@ def path_inject() -> None:
     # This must be run before we do run any subprocesses, and loading config
     # does this as part of the ansible detection.
     paths = [x for x in os.environ.get("PATH", "").split(os.pathsep) if x]
+
+    # Expand ~ in PATH as it known to break many tools
+    expanded = False
+    for idx, path in enumerate(paths):
+        if "~" in path:
+            paths[idx] = os.path.expanduser(path)
+            expanded = True
+    if expanded:
+        print(
+            "WARNING: PATH altered to expand ~ in it. Read https://stackoverflow.com/a/44704799/99834 and correct your system configuration.",
+            file=sys.stderr,
+        )
+
     inject_paths = []
 
     userbase_bin_path = f"{site.getuserbase()}/bin"
@@ -348,6 +391,7 @@ def path_inject() -> None:
             f"WARNING: PATH altered to include {', '.join(inject_paths)} :: This is usually a sign of broken local setup, which can cause unexpected behaviors.",
             file=sys.stderr,
         )
+    if inject_paths or expanded:
         os.environ["PATH"] = os.pathsep.join([*inject_paths, *paths])
 
     # We do know that finding ansible in PATH does not guarantee that it is
@@ -369,7 +413,7 @@ def to_bool(value: Any) -> bool:
     return False
 
 
-def should_do_markup(stream: TextIO = sys.stdout) -> bool:
+def should_do_markup(stream: TextIO = sys.stdout) -> bool:  # pragma: no cover
     """Decide about use of ANSI colors."""
     py_colors = None
 
