@@ -1,6 +1,7 @@
 """Application."""
 from __future__ import annotations
 
+import itertools
 import logging
 import os
 from functools import lru_cache
@@ -84,7 +85,7 @@ class App:
         # If run under GitHub Actions we also want to emit output recognized by it.
         if os.getenv("GITHUB_ACTIONS") == "true" and os.getenv("GITHUB_WORKFLOW"):
             formatter = formatters.AnnotationsFormatter(self.options.cwd, True)
-            for match in matches:
+            for match in itertools.chain(fatal_matches, ignored_matches):
                 console.print(formatter.format(match), markup=False, highlight=False)
 
     def count_results(self, matches: list[MatchError]) -> SummarizedResults:
@@ -193,7 +194,7 @@ warn_list:  # or 'skip_list' to silence them completely
                 "because 'yaml' is in 'skip_list'."
             )
 
-        if summary.failures:
+        if mark_as_success and summary.failures and not self.options.progressive:
             mark_as_success = False
 
         if not self.options.quiet:
@@ -319,13 +320,25 @@ def _sanitize_list_options(tag_list: list[str]) -> list[str]:
     return sorted(set(tags))
 
 
-@lru_cache(maxsize=1)
-def get_app(offline: bool = False) -> App:
-    """Return the application instance."""
+@lru_cache
+def get_app() -> App:
+    """Return the application instance, caching the return value."""
+    offline = default_options.offline
     app = App(options=default_options)
     # Make linter use the cache dir from compat
     default_options.cache_dir = app.runtime.cache_dir
 
-    app.runtime.prepare_environment(install_local=True, offline=offline)
+    role_name_check = 0
+    if "role-name" in app.options.warn_list:
+        role_name_check = 1
+    elif "role-name" in app.options.skip_list:
+        role_name_check = 2
+
+    # mocking must happen before prepare_environment or galaxy install might
+    # fail.
     _perform_mockings()
+    app.runtime.prepare_environment(
+        install_local=True, offline=offline, role_name_check=role_name_check
+    )
+
     return app
