@@ -55,7 +55,16 @@ from ansiblelint._internal.rules import (
 )
 from ansiblelint.app import get_app
 from ansiblelint.config import options
-from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS, FileType
+from ansiblelint.constants import (
+    FILENAME_KEY,
+    INCLUSION_ACTION_NAMES,
+    LINE_NUMBER_KEY,
+    NESTED_TASK_KEYS,
+    PLAYBOOK_TASK_KEYWORDS,
+    ROLE_IMPORT_ACTION_NAMES,
+    SKIPPED_RULES_KEY,
+    FileType,
+)
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, discover_lintables
 from ansiblelint.skip_utils import is_nested_task
@@ -107,14 +116,39 @@ def ansible_templar(basedir: str, templatevars: Any) -> Templar:
 def ansible_template(
     basedir: str, varname: Any, templatevars: Any, **kwargs: Any
 ) -> Any:
-    """Render a templated string."""
+    """Render a templated string by mocking missing filters."""
     templar = ansible_templar(basedir=basedir, templatevars=templatevars)
-    return templar.template(varname, **kwargs)
+    # pylint: disable=unused-variable
+    for i in range(3):
+        try:
+            kwargs["disable_lookups"] = True
+            return templar.template(varname, **kwargs)
+        except AnsibleError as exc:
+            if (
+                "was found, however lookups were disabled from templating"
+                in exc.message
+            ):
+                # ansible core does an early exit when disable_lookup=True but
+                # this happens after the jinja2 syntax already passed.
+                break
+            if (
+                exc.message.startswith("template error while templating string:")
+                and "'" in exc.message
+            ):
+                missing_filter = exc.message.split("'")[1]
+                if missing_filter == "end of print statement":
+                    raise
+                # Mock the filter to avoid and error from Ansible templating
+                # pylint: disable=protected-access
+                templar.environment.filters._delegatee[missing_filter] = lambda x: x
+                # Record the mocked filter so we can warn the user
+                if missing_filter not in options.mock_filters:
+                    _logger.debug("Mocking missing filter %s", missing_filter)
+                    options.mock_filters.append(missing_filter)
+                continue
+            raise
 
 
-LINE_NUMBER_KEY = "__line__"
-FILENAME_KEY = "__file__"
-
 BLOCK_NAME_TO_ACTION_TYPE_MAP = {
     "tasks": "task",
     "handlers": "handler",
@@ -156,7 +190,7 @@ def _playbook_items(pb_data: AnsibleBaseYAMLObject) -> ItemsView:  # type: ignor
 
     # "if play" prevents failure if the play sequence contains None,
     # which is weird but currently allowed by Ansible
-    # https://github.com/ansible-community/ansible-lint/issues/849
+    # https://github.com/ansible/ansible-lint/issues/849
     return [item for play in pb_data if play for item in play.items()]  # type: ignore
 
 
@@ -185,7 +219,7 @@ def find_children(lintable: Lintable) -> list[Lintable]:  # noqa: C901
     basedir = os.path.dirname(str(lintable.path))
     # playbook_ds can be an AnsibleUnicode string, which we consider invalid
     if isinstance(playbook_ds, str):
-        raise MatchError(filename=str(lintable.path), rule=LoadingFailureRule())
+        raise MatchError(filename=lintable, rule=LoadingFailureRule())
     for item in _playbook_items(playbook_ds):
         # if lintable.kind not in ["playbook"]:
         #     continue
@@ -215,6 +249,7 @@ def template(
     basedir: str,
     value: Any,
     variables: Any,
+    fail_on_error: bool = False,
     fail_on_undefined: bool = False,
     **kwargs: str,
 ) -> Any:
@@ -230,7 +265,8 @@ def template(
         # I guess the filter doesn't like empty vars...
     except (AnsibleError, ValueError, RepresenterError):
         # templating failed, so just keep value as is.
-        pass
+        if fail_on_error:
+            raise
     return value
 
 
@@ -275,15 +311,11 @@ def _include_children(
     basedir: str, k: str, v: Any, parent_type: FileType
 ) -> list[Lintable]:
     # handle special case include_tasks: name=filename.yml
-    if (
-        k in ["include_tasks", "ansible.builtin.include_tasks"]
-        and isinstance(v, dict)
-        and "file" in v
-    ):
+    if k in INCLUSION_ACTION_NAMES and isinstance(v, dict) and "file" in v:
         v = v["file"]
 
     # we cannot really parse any jinja2 in includes, so we ignore them
-    if "{{" in v:
+    if "{{" in v:  # pragma: no branch
         return []
 
     if "import_playbook" in k and COLLECTION_PLAY_RE.match(v):
@@ -330,13 +362,7 @@ def _taskshandlers_children(
             results.append(children)
             continue
 
-        including_commands = (
-            "import_role",
-            "ansible.builtin.import_role",
-            "include_role",
-            "ansible.builtin.include_role",
-        )
-        if any(x in task_handler for x in including_commands):
+        if any(x in task_handler for x in ROLE_IMPORT_ACTION_NAMES):
             # lgtm [py/unreachable-statement]
             task_handler = normalize_task_v2(task_handler)
             _validate_task_handler_action_for_role(task_handler["action"])
@@ -379,22 +405,12 @@ def _get_task_handler_children_for_tasks_or_playbooks(
     child_type = k if parent_type == "playbook" else parent_type
 
     # Include the FQCN task names as this happens before normalize
-    task_include_keys = (
-        "include",
-        "include_tasks",
-        "import_playbook",
-        "import_tasks",
-        "ansible.builtin.include",
-        "ansible.builtin.include_tasks",
-        "ansible.builtin.import_playbook",
-        "ansible.builtin.import_tasks",
-    )
-    for task_handler_key in task_include_keys:
+    for task_handler_key in INCLUSION_ACTION_NAMES:
 
         with contextlib.suppress(KeyError):
 
             # ignore empty tasks
-            if not task_handler:
+            if not task_handler:  # pragma: no branch
                 continue
 
             file_name = task_handler[task_handler_key]
@@ -402,10 +418,15 @@ def _get_task_handler_children_for_tasks_or_playbooks(
                 file_name = file_name["file"]
 
             f = path_dwim(basedir, file_name)
+            while basedir not in ["", "/"]:
+                if os.path.exists(f):
+                    break
+                basedir = os.path.dirname(basedir)
+                f = path_dwim(basedir, file_name)
             return Lintable(f, kind=child_type)
 
     raise LookupError(
-        f'The node contains none of: {", ".join(task_include_keys)}',
+        f'The node contains none of: {", ".join(sorted(INCLUSION_ACTION_NAMES))}',
     )
 
 
@@ -459,18 +480,18 @@ def _rolepath(basedir: str, role: str) -> str | None:
         path_dwim(basedir, os.path.join("..", role)),
     ]
 
-    for loc in get_app(offline=True).runtime.config.default_roles_path:
+    for loc in get_app().runtime.config.default_roles_path:
         loc = os.path.expanduser(loc)
         possible_paths.append(path_dwim(loc, role))
 
     possible_paths.append(path_dwim(basedir, ""))
 
-    for path_option in possible_paths:
+    for path_option in possible_paths:  # pragma: no branch
         if os.path.isdir(path_option):
             role_path = path_option
             break
 
-    if role_path:
+    if role_path:  # pragma: no branch
         add_all_plugin_dirs(role_path)
 
     return role_path
@@ -481,7 +502,7 @@ def _look_for_role_files(
 ) -> list[Lintable]:
     # pylint: disable=unused-argument # main
     role_path = _rolepath(basedir, role)
-    if not role_path:
+    if not role_path:  # pragma: no branch
         return []
 
     results = []
@@ -511,7 +532,7 @@ def _sanitize_task(task: dict[str, Any]) -> dict[str, Any]:
     result = task.copy()
     # task is an AnsibleMapping which inherits from OrderedDict, so we need
     # to use `del` to remove unwanted keys.
-    for k in ["skipped_rules", FILENAME_KEY, LINE_NUMBER_KEY]:
+    for k in [SKIPPED_RULES_KEY, FILENAME_KEY, LINE_NUMBER_KEY]:
         if k in result:
             del result[k]
     return result
@@ -619,23 +640,23 @@ def task_to_str(task: dict[str, Any]) -> str:
     action = task.get("action")
     if isinstance(action, str) or not isinstance(action, dict):
         return str(action)
-    args = " ".join(
-        [
-            f"{k}={v}"
-            for (k, v) in action.items()
-            if k
-            not in [
-                "__ansible_module__",
-                "__ansible_module_original__",
-                "__ansible_arguments__",
-                LINE_NUMBER_KEY,
-                FILENAME_KEY,
-            ]
+    args = [
+        f"{k}={v}"
+        for (k, v) in action.items()
+        if k
+        not in [
+            "__ansible_module__",
+            "__ansible_module_original__",
+            "__ansible_arguments__",
+            LINE_NUMBER_KEY,
+            FILENAME_KEY,
         ]
-    )
+    ]
+
     for item in action.get("__ansible_arguments__", []):
-        args += f" {item}"
-    return f"{action['__ansible_module__']} {args}"
+        args.append(str(item))
+
+    return f"{action['__ansible_module__']} {' '.join(args)}"
 
 
 def extract_from_list(
@@ -683,27 +704,13 @@ def get_action_tasks(data: AnsibleBaseYAMLObject, file: Lintable) -> list[Any]:
     # Add sub-elements of block/rescue/always to tasks list
     tasks.extend(extract_from_list(tasks, NESTED_TASK_KEYS, recursive=True))
 
-    # Include the FQCN task names as this happens before normalize
-    return [
-        task
-        for task in tasks
-        if {
-            "include",
-            "include_tasks",
-            "import_playbook",
-            "import_tasks",
-            "ansible.builtin.include",
-            "ansible.builtin.include_tasks",
-            "ansible.builtin.import_playbook",
-            "ansible.builtin.import_tasks",
-        }.isdisjoint(task.keys())
-    ]
+    return tasks
 
 
 @lru_cache(maxsize=None)
 def parse_yaml_linenumbers(  # noqa: max-complexity: 12
     lintable: Lintable,
-) -> AnsibleBaseYAMLObject | AnsibleBaseYAMLObject:
+) -> AnsibleBaseYAMLObject:
     """Parse yaml as ansible.utils.parse_yaml but with linenumbers.
 
     The line numbers are stored in each node's LINE_NUMBER_KEY key.
@@ -747,7 +754,11 @@ def parse_yaml_linenumbers(  # noqa: max-complexity: 12
             if data is None:
                 break
             result.append(data)
-    except (yaml.parser.ParserError, yaml.scanner.ScannerError) as exc:
+    except (
+        yaml.parser.ParserError,
+        yaml.scanner.ScannerError,
+        yaml.constructor.ConstructorError,
+    ) as exc:
         raise RuntimeError("Failed to load YAML file") from exc
 
     if len(result) == 0:
@@ -856,6 +867,10 @@ def get_lintables(
                 _logger.debug("Ignored %s due to: %s", path, exc)
                 continue
 
+            if path.is_symlink() and not path.exists():
+                _logger.warning("Ignored broken symlink %s -> %s", path, path.resolve())
+                continue
+
             lintables.append(Lintable(path))
 
         # stage 2: guess roles from current lintables, as there is no unique
@@ -873,7 +888,7 @@ def _extend_with_roles(lintables: list[Lintable]) -> None:
             role = lintable.path
             while role.parent.name != "roles" and role.name:
                 role = role.parent
-            if role.exists and not role.is_file():
+            if role.exists() and not role.is_file():
                 lintable = Lintable(role, kind="role")
                 if lintable not in lintables:
                     _logger.debug("Added role: %s", lintable)
